package com.isu.gaswellwatch.modbus.data.decode.impl;

import cn.hutool.core.map.MapUtil;
import com.isu.gaswellwatch.modbus.data.ModbusMessage;
import com.isu.gaswellwatch.modbus.data.decode.DecodeHandler;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.math.NumberUtils;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.Map;

/**
 * 乘除系数解析器
 * <ul>
 *     <ol>系数为正数时，采用除法</ol>
 *     <li>例如：输入值：2580，系数(factor)：100，精度(precision)：2，输出值：2580 / 100 = 25.8</li>
 *     <ol>系数为负数时，采用乘法</ol>
 *      <li>例如：输入值：2580，系数(factor)：100，精度(precision)：2，输出值：2580 * 100 = 258000</li>
 * </ul>
 *
 * @author <a href="mailto:scwsl@foxmail.com">王仕龙</a>
 * 2024/11/24 21:23
 */
@Component(FactorDecodeHandler.NAME + DecodeHandler.DECODE_NAME)
public class FactorDecodeHandler implements DecodeHandler {
    public static final String NAME = "factor";


    @Override
    public String decode(Map<String, Object> commandPointMap, String value) {
        String factorStr = MapUtil.getStr(commandPointMap, "factor");
        if (StringUtils.isNotBlank(factorStr) && NumberUtils.isCreatable(value)) {
            BigDecimal factor = new BigDecimal(factorStr);
            BigDecimal numberValue = new BigDecimal(value);
            int precision = MapUtil.getInt(commandPointMap, "precision", 0);
            switch (factor.compareTo(BigDecimal.ZERO)) {
                case -1 -> {
                    return numberValue.multiply(factor.abs()).toString();
                }
                case 1 -> {
                    return numberValue.divide(factor, precision, RoundingMode.HALF_UP).toString();
                }
            }
        }
        return value;
    }

    @Override
    public void decode(Map<String, Object> commandPointMap, ModbusMessage.MessagePoint point) {
        point.setValue(point.getParseValue());
        String factorStr = MapUtil.getStr(commandPointMap, "factor");
        if (StringUtils.isNotBlank(factorStr) && NumberUtils.isCreatable(point.getParseValue())) {
            BigDecimal factor = new BigDecimal(factorStr);
            BigDecimal value = new BigDecimal(point.getValue());
            int precision = MapUtil.getInt(commandPointMap, "precision", 0);
            switch (factor.compareTo(BigDecimal.ZERO)) {
                case -1 -> point.setValue(value.multiply(factor.abs()).toString());
                case 1 -> point.setValue(value.divide(factor, precision, RoundingMode.HALF_UP).toString());
            }
        }
    }

}
